################################################################################
# File    : hsm2cpp.tcl
# Content : a Tcl script to generate C++ code from a hierachical state machine
#           definition
# History : 09-Jan-2002 - [rb] creation
#           12-Feb-2002 - [rb] completed parsing
#           19-Feb-2002 - [rb] completed code generation
#           01-Mar-2002 - [rb] added send in actions
#                              added input declaration
#           04-Mar-2002 - [rb] added escape action
#           29-Apr-2002 - [rb] added the ability to repost a message after
#                              triggering a transition
#           11-Jun-2002 - [rb] added the persistance
#           01-Jul-2002 - [rb] corrected a bug in destructor (should not delete
#                              entries).
#           05-Jul-2002 - [rb] added the ability to declare variable with
#                              initial value like this:
#                              var <name>(<ctor params>)
#           05-Jul-2002 - [rb] corrected an error in destructor (removeReceiver
#                              is not neccessary).
#           11-Sep-2002 - [rb] added the ability to choose the initial hsm
#                              depending on the context
#           20-Sep-2002 - [rb] changed the var semantic:
#                              an uninitialised variable get its value from
#                              the constructor.
#           26-Sep-2002 - [rb] added the ability to declare methods:
#                              virtual <type> <name>(<args>) <qual> {<code>}
#           03-Mar-2003 - [rb] changed code generation to match a new hsm
#                              implementation
#           07-Mar-2003 - [rb] added transition triggered by method invocation:
#                              on doIt() [with ...] { ...} [-> NextHsm]
#           09-Apr-2003 - [rb] BugFix lsearch interpret * pattern as matching
#                              any thing if -exact is not specified
#           13-Jun-2003 - [rb] added init { code }rule
#           31-Jan-2004 - [rb] added the ability to have variables that are
#                              references.
#                              added the ability to change inputs on the fly
#           04-Feb-2004 - [rb] added the ability to assert condition before
#                              entering a state
#           09-Feb-2004 - [rb] splitted hsm2cpp.tcl into parser.tcl,
#                              hsm2ast.tcl, ast2cpp.tcl & hsm2cpp.tcl
#           13-Jul-2004 - [rb] added local method
#           16-Jul-2004 - [rb] removed code duplication when a hsm is used 
#                              multiple times
#           01-Sep-2004 - [rb] added argument support to events trigered by a 
#                              method
#           25-Oct-2004 - [rb] added generic event transition:
#                              on * from ...
#           10-Nov-2004 - [rb] changed and unified transition syntax.
#                              see hsm2ast.tcl parseAction
################################################################################

#   Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# ToDo (-) / Done (+) ##########################################################
# + add the ability to transmit the triggered event on a transition to the new
#   state (-> normal transition, => transition with repost)
# + add the persistance ( -h-> , =h=> )
#   The initial state of the entered machine is the one which was active when
#   the machine was left.
# + add the ability to choose the initial state with the context.
#   [condition] : InitialHsm
#   rq : what should be considered first, history or initial state ???
# + make the methods declared with virtual.
# - better error message (just the last line and its number)
# + rewrite parseSend (if top()->dispatch is not accessible by subHsms)
# + add "method events" for transitions (on enable() {} -> ...)
# - precompute transitions
# - should a hsm input be usable by subhsm as a variable ?
# - check if the type of variable that is a super hsm input is hsm::Component &
# - add an automatic typing (if a var or input as already be used somewhere
#   else)
# + support references variables
# + dynamically change the inputs
# - preserve indentation while copying code
# + allow transition that span betwen multiple levels
# - do variable aliasing using name and type information
# + guard on timer transition


# globals ######################################################################

set input ""
set parsed ""
array set hsmDefinition {}


set index(name)       0
set index(init)       1
set index(enter)      2
set index(leave)      3
set index(timer)      4
set index(actions)    5
set index(subHsms)    6
set index(outInputs)  7
set index(intInputs)  8
set index(outVars)    9
set index(intVars)   10
set index(initials)  11
set index(virtuals)  12
set index(locals)    13
set index(hsmInputs) 14
set index(methods)   15
set index(asserts)   16



# sources ######################################################################

set path [file dirname $argv0]
source [file join $path parser.tcl]
source [file join $path hsm2ast.tcl]
source [file join $path ast2cpp.tcl]



# main #########################################################################


# hsm2cpp --
#
#   Translate HSM definition into C++
#
# parameters
#   hsmname is the name of the .hsm file to parse

proc hsm2cpp {hsmname} {
	global input
	global parsed

	set verb_h   ""
	set verb_cpp ""

	set body_h   ""
	set body_cpp ""


	# reading input file
	set in [open $hsmname r]
	set input [string trimright [read $in]]
	close $in

	# skipping comments and white lines
	regsub -all -line -- {//.*} $input {} input
	regsub -all -- {/\*([^/]|[^*]/)*\*/} $input {} input
	regsub -all -- {([ \t]*\n)([ \t]*\n)+} $input "\n\n" input

	# including hsm files
	set b [string first "#include" $input]
	while {$b >= 0} {
		set fileName ""
		regexp -start [expr $b+8] -- {"([^"]*)"|<[^>]*>} $input _ fileName

		if {[string range $fileName end-3 end] == ".hsm"} {
			set in [open [file join [file dirname $hsmname] $fileName] r]
			set content [read $in]
			close $in

			# skipping comments and white lines
			regsub -all -line -- {//.*} $content {} content
			regsub -all -- {/\*([^/]|[^*]/)*\*/} $content {} content
			regsub -all -- {([ \t]*\n)([ \t]*\n)+} $content "\n\n" content

			set e [expr [string first $fileName $input $b] + [string length $fileName]]
			set input [string replace $input $b $e $content]
		}

		set b [string first "#include" $input [expr $b + 1]]
	}


	# get verbatim header
	set line [getLine]
	while {([string first "hsm" $line] != 0) & ($input != "")} {
		append verb_h "$line\n"
		set line [getLine]
	}
	putBackLine
	

	# parse the hsm
	set token [getWord]
	while {([lsearch {"hsm"} $token] >= 0)} {
		set hsm [parseHsm]
		set hsm [genHsm $hsm]
		
		append body_h   [lindex $hsm 0]
		append body_cpp [lindex $hsm 1]
		
		set token [getToken]
	}

	# get verbatim implementation
	while {$token != "__EOF__"} {
		append verb_cpp "$token"
		set token [getWord]
	}


	set name [file rootname [lindex [file split $hsmname] end]]
	set date [string toupper [clock format [clock seconds] -format "%d-%b-%Y"]]


	### writing header ###

	set def  [string toupper $name]
	regsub -all -- {[^A-Z]} $def "_" def
	append def _H

	set header_h "////////////////////////////////////////////////////////////////////////////////
// File    : $name.h
//           generated from $name.hsm
// Content : header of c++ $name HSM
// History : $date - \[hsm2cpp.tcl\] automatic creation
////////////////////////////////////////////////////////////////////////////////


#if !defined($def)
#define $def\n\n\n"

	set verb_h [string trim $verb_h]
	if {$verb_h != ""} {
		append header_h "/* part from original file ****************************************************/\n\n"
		append header_h "$verb_h\n\n\n"
	}

	append header_h "/* includes *******************************************************************/\n\n"
	append header_h "#include <hsm/component/Hsm.h>\n\n"


	set footer_h "\n#endif // !defined($def)\n"

	set out [open $name.h w]
	puts -nonewline $out $header_h
	puts -nonewline $out $body_h
	puts -nonewline $out $footer_h
	close $out


	### writing implementation ###

	set header_cpp "////////////////////////////////////////////////////////////////////////////////
// File    : $name.cpp
//           generated from $name.hsm
// Content : implementation of $name HSM
// History : $date - \[hsm2cpp.tcl\] automatic creation
////////////////////////////////////////////////////////////////////////////////


/* includes *******************************************************************/\n\n"

	append header_cpp "#include \"$name.h\"\n\n\n"
	
	set verb_cpp [string trim $verb_cpp]
	if {$verb_cpp != ""} { 
		append header_cpp "/* part from original file ****************************************************/\n\n"
		append header_cpp "$verb_cpp\n\n\n"
	}
	
	set out [open $name.cpp w]
	puts -nonewline $out $header_cpp
	puts -nonewline $out $body_cpp
	close $out

	return -code ok
}


if [catch {hsm2cpp $argv}] {
	puts stderr "Usage : tclsh hsm2cpp.tcl <file.hsm>"
	exit 1
}
