#include <sstream>

#include <w3c/svg/SVGStylable.hpp>
#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGRectElement.hpp>
#include <w3c/svg/SVGCircleElement.hpp>

#include <hsm/interaction/Protocol.h>
#include <hsm/svgl/SVGLBehaviour.h>

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/Svgl.h>

#include <Protocol.h>


/* forward declarations *******************************************************/

class Circle;


/* CircleHandle class declaration *********************************************/

class CircleHandle : public hsm::SVGLBehaviour, public hsm::Translate, public hsm::EnterLeave {
	static const hsm::SVGLBehaviour::Factory::Register< CircleHandle > registration;

protected :
	svg::SVGStylable *handle;
	Circle *circle;
	
public :
	CircleHandle(svg::SVGElement *e, hsm::SVGLWindow *w);
	
	virtual bool begin();
	virtual void end();
	
	virtual void enter();
	virtual void leave();	

	virtual void translate(const hsm::Point &p);
};


/* Rectangle class declaration ************************************************/

class Circle : public hsm::SVGLBehaviour, public hsm::Translate, public RaiseLower, public Deletable, public Colorable {
	static const hsm::SVGLBehaviour::Factory::Register< Circle > registration;
	
	svg::SVGCircleElement *circle;
	svg::SVGRectElement *handle;

public :
	Circle(svg::SVGElement *e, hsm::SVGLWindow *w);
	virtual ~Circle();
	
	void resize(float dx, float dy, float dr);
	
	virtual void translate(const hsm::Point &p);
	virtual void raise();
	virtual void lower();
	virtual void setColor(float r, float g, float b);
};



/* CircleHandle implementation ************************************************/

CircleHandle::CircleHandle(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLBehaviour(e, w) {
	handle = dynamic_cast< svg::SVGStylable * >(elem);

	handle->setFill("black");
	handle->setOpacity(0.);

	hsm::svgl::getAttribute(elem, "hsm-arg:parent", circle);
}

bool CircleHandle::begin() {
	return circle->begin();
}

void CircleHandle::end() {
	circle->end();
}

void CircleHandle::enter() {
	window->lock();
	handle->setOpacity(.5);
	window->unlock();
}

void CircleHandle::leave() {
	window->lock();
	handle->setOpacity(0.);
	window->unlock();
}

void CircleHandle::translate(const hsm::Point &p) {
	circle->resize(0., 0., p/hsm::Point::X);
}

const hsm::SVGLBehaviour::Factory::Register< CircleHandle > CircleHandle::registration("handle");


/* Circle implementation ******************************************************/

Circle::Circle(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLBehaviour(e, w) {
	// main object 
	circle = dynamic_cast< svg::SVGCircleElement * >(elem); // TODO : check if circle != 0
	
	// handles
	float X = circle->getCx().getValue();
	float Y = circle->getCy().getValue();
	float R = circle->getR().getValue();

	handle = new svg::SVGRectElement(window->svgDocument);
	handle->setX(X + R - 5);
	handle->setY(Y - 5);
	handle->setHeight(9);
	handle->setWidth(9);
	window->svgFeedback->appendChild(handle);
	hsm::svgl::setAttribute(handle, "hsm:behaviour", "handle");
	hsm::svgl::setAttribute(handle, "hsm-arg:parent", this);
	window->instanciateBehaviour(handle);
}

Circle::~Circle() {
	delete hsm::SVGLBehaviour::Factory::get(handle);
	window->lock();
	elem->getParentNode()->removeChild(elem);
	window->svgFeedback->removeChild(handle);
	window->unlock();
}


void Circle::resize(float dx, float dy, float dr) {
	window->lock();
	float X = circle->getCx().getValue() + dx;
	float Y = circle->getCy().getValue() + dy;
	float R = circle->getR().getValue() + dr;

	circle->setCx(X);
	circle->setCy(Y);
	circle->setR(R);

	handle->setX(X + R - 5);
	handle->setY(Y - 5);

	window->unlock();
}

void Circle::translate(const hsm::Point &p) {
	resize(p/hsm::Point::X, p/hsm::Point::Y, 0.);
}

void Circle::raise() {
	circle->raise();
	handle->raise();
}

void Circle::lower() {
	circle->lower();
}

void Circle::setColor(float r, float g, float b) {
	std::stringstream color;
	color << "rgb(" << int(r) << "," << int(g) << "," << int(b) << ")";
	circle->setFill(color.str().c_str());
}


const hsm::SVGLBehaviour::Factory::Register< Circle > Circle::registration("circle");
