////////////////////////////////////////////////////////////////////////////////
// File    : Processing.h
// Content : processing policies for hsm component
// History : 08-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file Processing.h
/// Processing policies for libhsm components.


// ToDo(-) / Done(+) ///////////////////////////////////////////////////////////
// - check that Base class inherits from hsm::Component


#if !defined(HSM_PROCESSING_H)
#define HSM_PROCESSING_H


/* includes *******************************************************************/

#include <hsm/core/Event.h>
#include <hsm/system/thread/Mutex.h>
#include <hsm/system/select/Alert.h>
#include <hsm/system/select/Selector.h>
#include <hsm/util/Functor.h>
#include <hsm/util/Template.h>

#include <queue>


namespace hsm {

	/// Eventprocessing policies
	namespace processing {

/* Sync processing policy declaration *****************************************/

		/// handles the event synchronously when they are received.
		/// The Base class should inherit from hsm::Component.
		template< class Base >
		class Sync : public Base {
			COMPILE_ASSERT(BASE_OF(hsm::Component, Base), Base_should_inherite_from_Component);
			
		public :
			/// constructor
			/// \param id component id
			Sync(const typename Base::Id &id) : Base(id) {}
			
		private :
			virtual void doPost(const Event &event) { process(event); }
		};


/* Async processing policy declaration ****************************************/

		/// handles the event asynchronously when they are received.
		/// The events are enqueued and asynchronously processed in another thread.
		/// The Base class should inherit from hsm::Component.
		template< class Base >
		class Async : public Sync< Base > {
			typedef std::queue< Event > EventQueue;

		protected :
			Mutex    mutex;
			Alert    alert;
			Selector selector;
			Functor *processor;

			EventQueue eventQueue;

		public :
			/// constructor
			/// \param id component id
			Async(const typename Base::Id &id) : Sync< Base >(id) {
				processor = Functor::create(this, &Async::processEvents);
				selector.add(alert, Selector::R, processor);
			}

			/// destructor
			virtual ~Async() {
				selector.remove(alert, Selector::R, processor);
				delete processor;
			}

		private :
			virtual void doPost(const Event &event) {
				mutex.lock();
				eventQueue.push(event);
				mutex.unlock();
				alert.set();
			}

			void processEvents() {
				while(!eventQueue.empty()) {
					const Event &event = eventQueue.front();
					process(event);
					mutex.lock();
					eventQueue.pop();
					mutex.unlock();
					alert.reset();
				}
			}

		};


/* MonoAsync processing policy declaration ************************************/

		/// handles the event asynchronously when they are received and compress the queue if to following events are identical.
		/// The events are enqueued and asynchronously processed in another thread.
		/// The Base class should inherit from hsm::Component.
		template< class Base >
		class MonoAsync : public Async< Base > {
		public :
			/// constructor
			/// \param id component id
			MonoAsync(const typename Base::Id &id) : Async< Base >(id) {}

		private :
			virtual void doPost(const Event &event) {
				bool added = false;
				mutex.lock();
				if(eventQueue.empty()) {
					eventQueue.push(event);
					added = true;
				} else {
					const Event &lastEvent = eventQueue.back();
					if((lastEvent.sender != event.sender) || (lastEvent.message != event.message)) {
						eventQueue.push(event);
						added = true;
					}
				}
				mutex.unlock();
				if(added) alert.set();
			}
			
		};


/* Callback vs Functor policy declaration *************************************/

		/// type of processing policies
		enum {
			Callback, ///< event are processed by the callback
			Functor   ///< event are processed by a functor
		};


	} // namespace processing 

} // namespace hsm


#endif // !defined(HSM_PROCESSING_H)
