////////////////////////////////////////////////////////////////////////////////
// File    : SVGLBehaviour.h
// Content : interaction protocols implementation for svgl objects
// History : 26-Aug-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#if !defined(HSM_SVGLBEHAVIOUR_H)
#define HSM_SVGLBEHAVIOUR_H


/* includes *******************************************************************/

#include <hsm/interaction/Protocol.h>

#include <string>
	

/* forward declarations ******************************************************/

namespace svg {
	class SVGDocument;
	class SVGElement;
	class SVGTranslateTransform;
	class SVGScaleTransform;
	class SVGTransformable;
	class SVGStylable;
}

namespace hsm {

	class SVGLWindow;


/* SVGLBehaviour class definition *********************************************/

	class SVGLBehaviour : virtual Protocol {
	public :
		class Factory {
			friend class SVGLBehaviour;
		
			typedef SVGLBehaviour *Constructor(svg::SVGElement *, SVGLWindow *);
			static const Factory *instance;	
			Factory();

		protected :
			typedef std::map< std::string, Constructor * > ConstructorMap;
			static ConstructorMap *constructors;
		
			typedef std::map< svg::SVGElement *, SVGLBehaviour * > BehaviourMap;
			static BehaviourMap *behaviours;
		
			template< class P >
			static SVGLBehaviour *construct(svg::SVGElement *e, SVGLWindow *w) {
				return new P(e, w);
			}
		
		public :
			static void ensureLoading();
		
			template< class Behaviour >
			struct Register {
				Register(const std::string &type) {
					ensureLoading();
					(*constructors)[type] = &construct< Behaviour >;
				}
			};
		
			static SVGLBehaviour *create(std::string type, svg::SVGElement *e, SVGLWindow *w);
			static SVGLBehaviour *get(const svg::SVGElement *e);
			static SVGLBehaviour *get(const svg::SVGDocument *doc, const std::string &id);
		};
	
	protected :
		svg::SVGElement *elem;
		SVGLWindow *window;
		
	public :
		SVGLBehaviour(svg::SVGElement *e, SVGLWindow *w);
		virtual ~SVGLBehaviour();
	};
	
	
/* SVGLEnterLeave protocol declaration ****************************************/

	class SVGLEnterLeave : public virtual SVGLBehaviour, public EnterLeave {
	protected :
		svg::SVGStylable *stylable;
		float strokeWidth;
		
	public :
		SVGLEnterLeave(svg::SVGElement *e, SVGLWindow *w);
		
		virtual void enter();
		virtual void leave();
	};
	
	
/* PushPop protocol declaration ***********************************************/

	class SVGLPushPop : public virtual SVGLBehaviour, public PushPop {
	public :
		SVGLPushPop(svg::SVGElement *e, SVGLWindow *w) : SVGLBehaviour(e, w) {}
		
		virtual void push() {};
		virtual void pop()  {};
	};
	
	
/* Translate protocol declaration *********************************************/

	class SVGLTranslate : public virtual SVGLBehaviour, public Translate {
	protected :
		float sx, sy, tx, ty;
		svg::SVGTranslateTransform *trans;
		
	public :
		SVGLTranslate(svg::SVGElement *e, SVGLWindow *w);
		
		virtual void translate(const Point &p);	
	};
	

/* Zoom protocol declaration **************************************************/

	class SVGLZoom : public virtual SVGLBehaviour, public Zoom {
	protected :
		float sx, sy, tx, ty;
		svg::SVGScaleTransform *scale;
		svg::SVGTranslateTransform *trans;
		
	public :
		SVGLZoom(svg::SVGElement *e, SVGLWindow *w);
		
		virtual void zoom(float ds = 1.);
	};
	
	
} // namespace hsm


#endif // !defined(HSM_SVGLBEHAVIOUR_H)
