////////////////////////////////////////////////////////////////////////////////
// File    : Selector.cpp
// Content : A manager for select operations
// History : 31-Jan-2003 - [rb] creation
//           08-Feb-2003 - [rb] rewrite, added add / remove
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2003  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <Selector.h>

#include <hsm/system/select/Alert.h>
#include <hsm/system/thread/Mutex.h>
#include <hsm/system/thread/ThreadLoop.h>
#include <hsm/util/Functor.h>


namespace hsm {

/* Selector implementation ****************************************************/

	Selector::Selector(unsigned int ms /* = 0 */) : timeOut(ms), statusMapMap() {
		registrationAlert = new Alert;
		statusMutex = new Mutex;
		listMutex = new Mutex;
		
		// launching the select thread
		selectorFn = Functor::create(this, &hsm::Selector::watch);
		threadLoop = new ThreadLoop(selectorFn);
		threadLoop->play();
	}
	
	Selector::~Selector() {
		threadLoop->stop();
		registrationAlert->set();

		delete threadLoop;
		delete selectorFn;

		delete listMutex;
		delete statusMutex;
		delete registrationAlert;
	}
	
	void Selector::add(file::Descriptor fd, Status status /* = R */, Functor *fn /* = 0 */) {
		statusMutex->lock();
		Status &s = statusMapMap[fd][fn];
		s = Status(s | status);
		statusMutex->unlock();
		registrationAlert->set();
	}
	
	void Selector::remove(file::Descriptor fd, Status status /* = RWE */, Functor *fn /* = 0 */) {
		statusMutex->lock();
		StatusMap &sm = statusMapMap[fd];
		Status &s = sm[fn];
		s = Status((unsigned int)(s) & !(unsigned int)(status));
		
		// removing empty things
		if(s == N) {
			sm.erase(fn);
			if(sm.empty()) {
				statusMapMap.erase(fd);
			}
		}
		statusMutex->unlock();
		registrationAlert->set();
	}
	

	void Selector::setTimeOut(unsigned int ms /* = 0 */) {
		timeOut = ms;
	}
	
	void Selector::select() {
		listMutex->lock();
		Mutex wait;
		wait.lock();
		waitingList.push_back(&wait);
		listMutex->unlock();

		// waiting till watch unlocks wait
		wait.lock();
	}


	void Selector::watch() {
		os::watch(this);
		listMutex->lock();
		for(std::list< Mutex * >::iterator m = waitingList.begin(); m != waitingList.end(); ++m) {
			(*m)->unlock();
		}
		waitingList.clear();
		listMutex->unlock();
	}
	

} // namespace hsm