////////////////////////////////////////////////////////////////////////////////
// File    : ThreadLoop.h
// Content : a thread based classes
// History : 19-Dec-2001 - [rb] creation
//           18-Jan-2002 - [rb] adapted to pass data to the Functor object
//           04-Sep-2002 - [rb] added a critical section object
//           02-Feb-2003 - [rb] change pause implementation to count
//                              play / pause pairs.
//           04-Feb-2003 - [rb] added mutex class and rewrite critical section
//           22-Oct-2003 - [rb] added ensureLoading
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file ThreadLoop.h
/// Classes to deal with threads and concurency.


#if !defined(HSM_THREADLOOP_H)
#define HSM_THREADLOOP_H


/* includes *******************************************************************/

#if !defined(WIN32)
#  include <pthread.h>
#  include <unistd.h>
#else
#  define _WIN32_WINNT 0x0400
#  include <windows.h>
#endif

#include <map>


namespace hsm {

/* forward declaration ********************************************************/

	class Functor;


/* ThreadLoop class declaration ***********************************************/

	/// Implements a thread abstraction.
	class ThreadLoop {

		// os dependent declaration
		struct os {
			// typedefs
		#if !defined(WIN32)
			typedef void             *Fn(void *); ///< pointer on thread function
			typedef pthread_t         Id;         ///< thread descriptor
		#else
			typedef DWORD WINAPI      Fn(LPVOID); ///< pointer on thread function
			typedef DWORD             Id;         ///< thread descriptor
		#endif

			// functions
			static Fn   threadFn;
			static Id   create(ThreadLoop *tl);
			static void waitFor(Id id);
			static Id   self();
		};

		friend struct os;

	private :
		typedef std::map< os::Id, ThreadLoop * > Map;
		static Map threadLoopMap;

	private :
		os::Id threadId;

		Functor *preamble;
		Functor *threadLoopFn;
		Functor *postamble;

		unsigned int delay;

		void *data;

		volatile bool stopping;
		volatile int  pausing;

	public :
		
		/// constructor
		/// \param f functor called repeadly until thread loop end
		/// \param ms time in ms between successive \c f calls (defaults to 0)
		/// \param d \c void pointer on user data passed to \c f (defaults to 0)
		/// \param pre preamble functor called once before the first \c f call (defaults to 0)
		/// \param post postamble functor called once after the last \c f call (defaults to 0)
		ThreadLoop(Functor *f, unsigned int ms = 0, void *d = 0, Functor *pre = 0, Functor *post = 0);

		/// destructor
		/// waits till the current iteration is finished
		virtual ~ThreadLoop();

		
		void play();  ///< start or restart the thread loop
		void pause(); ///< pause the thread loop after the current iteration
		void stop();  ///< stops the thread loop after the current iteration

		
		/// sets the delay between loop iterations
		/// \param d delay in ms
		void setDelay(unsigned int d);
		
		
		/// gets the thread loop of the calling code
		/// \return a pointer to the thread loop (0 if none)
		static ThreadLoop *get();
		
		/// waits till the thread loop is finished
		/// \param tl pointer on a thread loop
		static void waitFor(const ThreadLoop *tl);

	protected :
		/// preambles, loops, and postambles...
		void loop();
	};


} // namespace hsm


#endif // !defined(HSM_THREADLOOP_H)
