////////////////////////////////////////////////////////////////////////////////
// File    : GLWindow.h
// Content : Window class with openGL support
// History : 11-Oct-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file GLWindow.h
/// Supporting window with openGL.


#if !defined(HSM_GLWINDOW_H)
#define HSM_GLWINDOW_H


/* includes *******************************************************************/

#include <hsm/system/window/Window.h>

#if defined(MACOSX)
#  include <AGL/agl.h>
#  include <OpenGL/OpenGL.h>
#  include <OpenGL/gl.h>
#else
#  include <gl/gl.h>
#endif


namespace hsm {

/* GLWindow class definition **************************************************/

	/// implements a window with openGL support.
	/// init, display, and reshape should be overidden to suit particular needs.
	class GLWindow : public Window {

		/// system dependent functions
		struct os {
		#if defined(MACOSX)
			typedef AGLContext Context; ///< openGL context
		#elif defined(WIN32)
			typedef HGLRC      Context; ///< openGL context
		#endif
		
			/// creates a openGL context
			/// \param glw the window to support openGL
			/// \return the system openGL context
			static Context createGlContext(GLWindow *glw);

			/// destroys a openGL context
			/// \param glw the window
			static void    destroyGlContext(GLWindow *glw);
			
			/// makes the window current for openGL
			/// \param glw the window
			static void makeCurrent(GLWindow *glw);

			/// swaps the window buffers
			/// \param glw the window
			static void swapBuffers(GLWindow *glw);
			
			/// makes the window swap synchronized with screen refresh
			/// \param glw the window
			/// \param vbl the number of vertical blank line to wait for before swapping
			static void setVBLSync(GLWindow *glw, long vbl = 1);
			
			
			/// reshapes the window context
			/// \param glw the window to reshape
			static void reshape(GLWindow *glw);
		};
		
		friend struct os;

	protected :
		os::Context glContext;

		Mutex glMutex;

	public :
		/// constructor
		/// \param id the window id
		GLWindow(const Id &id = Id());

		virtual ~GLWindow(); ///< destructor
		
		virtual void show(bool s = true);

		/// makes the window swap synchronized with screen refresh
		/// \param vbl the number of vertical blank line to wait for before swapping
		void setVBLSync(long vbl = 1);

		inline void lock() {
			glMutex.lock();
		}
	
		inline void unlock() {
			glMutex.unlock();
			post();
		}
		
	protected :
		virtual void init();
		virtual void display();
		virtual void reshape(unsigned int width, unsigned int height);

		virtual void process(const Event &event);
		
		void doInit();
		void doDisplay();
		void doReshape();
	};


} // namespace hsm


#endif // !defined(HSM_GLWINDOW_H)