////////////////////////////////////////////////////////////////////////////////
// File    : Window.h
// Content : Window class
// History : 11-Oct-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file Window.h
/// Window abstraction.


#if !defined(HSM_WINDOW_H)
#define HSM_WINDOW_H


/* includes *******************************************************************/

#include <hsm/core/Composite.h>

#if defined(MACOSX)
#  include <map>
#  include <Carbon/Carbon.h>
#elif defined(WIN32)
#  include <list>
#  define _WIN32_WINNT 0x0400
#  include <windows.h>
#endif

#include <string>


namespace hsm {

/* forward declarations *******************************************************/

	class Point;
	class EventHandler;


/* Window class definition ****************************************************/

	/// implements a window abstraction
	class Window : public Composite {
	public :
	
		/// os dependent window handling functions
		struct os {
		#if defined(MACOSX)
			typedef WindowPtr     Window;                                         ///< window descriptor
			typedef EventTypeSpec EventSpec;                                      ///< system event
			typedef OSStatus      Handler(EventHandlerCallRef, EventRef, void *); ///< window event handler
			typedef std::map< const EventHandler *, EventHandlerRef > Handlers;   ///< window event handlers
		#elif defined(WIN32)
			typedef HWND          Window;                                         ///< window descriptor
			typedef unsigned int  EventSpec;                                      ///< system event
			typedef LONG WINAPI   Handler(HWND, UINT, WPARAM, LPARAM);            ///< window event handler
			typedef std::list< const EventHandler * > Handlers;                   ///< window event handlers
		#endif
			
			/// creates a system window
			/// \param hw pointer on the hsm::Window that creates the window
			/// \param title title of the window
			/// \param x position of the left border
			/// \param y position of the top border
			/// \param w window width
			/// \param h window height
			/// \return a system dependent window handle
			static Window create(hsm::Window *hw, const std::string &title, int x, int y, int w, int h);
			
			/// destroys a system window
			/// \param window system window to destroy handle
			static void destroy(Window window);


			/// sets a system window position
			/// \param window system window to move
			/// \param x position of the left border
			/// \param y position of the top border
			/// \param w window width
			/// \param h window height
			static void setPosition(Window window, int x, int y, int w, int h);

			/// sets a system window size
			/// \param window system window to move
			/// \param x position of the left border
			/// \param y position of the top border
			/// \param w window width
			/// \param h window height
			static void setSize(Window window, int x, int y, int w, int h);
			
			/// sets a system window title
			/// \param window system window to move
			/// \param title window title
			static void setTitle(Window window, const std::string &title);
			
			
			/// shows a system window
			/// \param window system window to show
			static void show(Window window);
			
			/// hides a system window
			/// \param window system window to hide			
			static void hide(Window window);
			
			/// maximizes a system window
			/// \param window system window to maximize
			static void zoom(Window window);
			
			/// reduces a system window
			/// \param window system window to reduce
			static void unzoom(Window window);
			
			static void showCursor(); ///< shows the system cursor
			static void hideCursor(); ///< hides the system cursor
			
			/// adds an event handler to a window
			/// \param hw a hsm::Window to add event handler
			/// \param eh hsm::EventHandler to add
			static void addHandler(hsm::Window *hw, const EventHandler *eh);

			/// removes an event handler to a window
			/// \param hw a hsm::Window to remove event handler
			/// \param eh hsm::EventHandler to remove
			static void removeHandler(hsm::Window *hw, const EventHandler *eh);
			
			typedef std::map< Window, hsm::Window * > Map;
			static Map windowMap;

		private :
			static Handler defaultHandler;
			static void setHandler(Window w);
		};
		
		friend struct os;

	protected :
		os::Window   window;
		os::Handlers handlers;

	protected :
		Point *position;
		Point *size;

	public :
	
		/// constructor
		/// \param id id of the window
		Window(const Id &id = Id());
		
		/// destructor
		virtual ~Window();

		/// sets the window position
		/// \param p position of the top left corner
		void setPosition(const Point &p);
		
		/// sets the window position
		/// \param x position of the left border
		/// \param y position of the top border
		void setPosition(int x, int y);
		
		/// sets the window size
		/// \param s size of the window content
		void setSize(const Point &s);
		
		/// sets the window size
		/// \param x window width
		/// \param y window height		
		void setSize(int x, int y);
		
		/// sets the window title
		/// \param title window title
		void setTitle(const std::string &title);
		
		
		/// shows the window
		/// \param s show / hide the window
		virtual void show(bool s = true);
		void hide();                       ///< hides the window
		
		/// zooms the window
		/// \param s zoom / unzoom the window
		void zoom(bool s = true);
		
		/// shows the system cursor
		/// \param s show / hide the cursor
		void showCursor(bool s = true) const;
		void hideCursor() const;              ///< hides the cursor
		

		/// converts a point from screen to window coordinates
		/// \param sp point in screen coordinates to convert
		/// \param wp resulting point in window coordinates
		void screen2window(const Point &sp, Point &wp) const;
		
		/// converts a point from window to screen coordinates
		/// \param wp point in window coordinates to convert
		/// \param sp resulting point in screen coordinates
		void window2screen(const Point &wp, Point &sp) const;
		

		/// adds an event handler to the window
		/// \param eh the hsm::EventHandler
		void addEventHandler(const EventHandler *eh);

		/// removes an event handler from the window
		/// \param eh the hsm::EventHandler
		void removeEventHandler(const EventHandler *eh);
	};


} // namespace hsm


#endif // !defined(HSM_WINDOW_H)